#!/usr/bin/env perl
#
# Dumplevel.pm
# 

my $version = 0.3;

package Dumplevel;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(dumplevel);

use Time::Local;

sub dumplevel {
  use constant SECS_PER_DAY => 86400;

  # Parameter Uebernehmen und pruefen
  my $L0_interval = shift @_;
  if ( $L0_interval !~ /^\d+$/ ) { die "dumplevel(): Parameter L0_interval falsch" }

  my $L1_interval = shift @_;;
  if ( $L1_interval !~ /^\d+$/ ) { die "dumplevel(): Parameter L1_interval falsch" }
  if ( $L1_interval >= $L0_interval ) { die "dumplevel(): Parameter L1_interval: Wert nicht plausibel" }

  my $L2_interval = shift @_;
  if ( $L2_interval !~ /^\d+$/ ) { die "dumplevel(): Parameter L2_interval falsch" }
  if ( $L2_interval >= $L1_interval ) { die "dumplevel(): Parameter L2_interval: Wert nicht plausibel" }

  my $dumpdates = shift(@_) || '/etc/dumpdates';

  
  my %monat = qw (Jan 1 Feb 2 Mar 3 Apr 4 May 5 Jun 6 Jul 7 Aug 8 Sep 9 Oct 10 Nov 11 Dec 12);
  my ($day, $month, $year) = (localtime)[3, 4, 5];
  my $epoch = timelocal(0, 0, 0, $day, $month + 1, $year + 1900);

  open FH, $dumpdates or die "dumplevel(): Kann die Datei $dumpdates nicht oeffnen\n";

  my $L0 = 0; my $L1 = 0; my $L2 = 0; my $x0 = 1; my $x1 = 1;
  my $L0_date;
  my $Ddd = 'Mon|Tue|Wed|Thu|Fri|Sat|Sun';
  my $Mmm = 'Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec';
  while (<FH>) {
    $_ =~ /(\d) ($Ddd) ($Mmm) ([ | \d]\d).{10}(20[0-9]{2})/;
    my $Level = $1;
    my $mm = $monat{$3};
    my $dd = $4;
    my $yy = $5;
    $dd =~ s/ /0/;
    my $dumpdate = timelocal(0, 0, 0, $dd, $mm, $yy);
    my $age = ($epoch - $dumpdate) / SECS_PER_DAY;
    
    # Programm beenden wenn der letzte dump neuer als der Level 2 Intervall ist.
    return undef if $age < $L2_interval;

    if ($Level == 0) {
      $x0 = 0;
      $L0_date = $dumpdate;
      if ($age >= $L0_interval) { $L0 = 1}
      }
    elsif ($Level == 1) {
      $x1 = 0;
      if ($age >= $L1_interval) { $L1 = 1 }
      }
    }
    
  close FH;

  return 0 if $L0 || $x0;
  return 1 if $L1;
  if ($x1) {
    return 1 if ((($epoch - $L0_date) / SECS_PER_DAY) >= $L1_interval);
    }
  return 2;
  }

1 ;

__END__

=head1 NAME

Dumplevel - Ermittelt den naechsten erforderlichen Dumplevel

=head1 SYNOPSIS

use Dumplevel;

$LEVEL = dumplevel($L0_interval, $L1_interval, $L2_interval, $dumpdates);

=head1 DESCRIPTION

Diese Funktion liest die Datei $dumpdates aus und ermittelt den naechsten
erforederlichen Dumplevel fuer das Backupprogramm "dump". Sie erwartet als
Argumente die Intervalle fuer Level 0, Level 1 und Level 2 Dumps in Tagen.
Wobei der Level 0 Intervall groesser als der Level 1 Intervall sein muss
und dieser wiederum groesser als der Level 2 Intervall. Der vierte Parameter
ist optional. Er erwartet die Datei in der die "Dumpdates" gespeichert sind
(Standard = /etc/dumpdates).
Wenn die Datei $dumpdates leer ist, wird mit einem Level 0 Dump begonnen.
Es wird dann je nach festgelegtem Level 2 Intervall ein Level 2 Dump
durchgefuehrt, ausser es steht ein Level 1 oder Level 0 Dump an. Fruehestens
nach den in $L1_interval festgelegten Tage wird ein Level 1 Dump durchgefuehrt.
Ein Level 0 Dump wird dann nach der in $L0_interval festgelegtan Anzahl von
Tagen durchgefuehrt.

=cut
